import org.jgrapht.*;
import org.jgrapht.graph.*;

import fdp.ForceDirectedPlacement;
import fdp.GraphConfiguration;
import fdp.Parameter;
import fdp.graph.Edge;
import fdp.graph.Vertex;

import javax.vecmath.Vector2d;

Graph<Vertex, Edge> graph = new SimpleGraph<Vertex, Edge>(Edge.class);

float scaleFactor = 1.0f;
float translateX = 0.0f;
float translateY = 0.0f;

void setup() {
  size(720, 720, FX2D);

  // create network graph
  JSONObject data = loadJSONObject("../data-clean.json");
  String username = data.getString("username");
  String timestamp = data.getString("timestamp");

  JSONObject followers = data.getJSONObject("followers");

  SimpleVertex root = new SimpleVertex(username);
  graph.addVertex(root);

  for (Object key : followers.keys()) {
    String name = (String)key;
    JSONObject user = followers.getJSONObject(name);
    JSONArray theirfollowers = user.getJSONArray("followers");

    int followersCount = user.getInt("followers_count");
    int followingCount = user.getInt("following_count");

    float metric = followingCount / (float)followersCount;
    if (metric < 0.5) {
      break;
    }
    println("Followers: " + followersCount + " Follows: " + followingCount);

    SimpleVertex vertex = new SimpleVertex(name);
    graph.addVertex(vertex);

    // connect root and node
    addEdge(root, vertex);

    int count = 0;

    for (String followerName : theirfollowers.getStringArray()) {
      SimpleVertex followerVertex = new SimpleVertex(followerName);
      graph.addVertex(followerVertex);
      addEdge(vertex, followerVertex);

      count++;

      if (count > 20) 
        break;
    }
  }

  println("Added " + graph.vertexSet().size() + " vertices...");

  // layout graph
  println("Layouting graph...");

  // layout graph
  Parameter p = new Parameter();
  p.setFrameWidth(width * 4);
  p.setFrameHeight(height * 4);
  p.setEquilibriumCriterion(false);
  p.setCriterion(8);

  p.setAttractiveForce("(d * d) / k");
  p.setRepulsiveForce("(k * k) / d");
  p.setCoolingRate(0.035);
  p.setFrameDelay(0);

  try {
    ForceDirectedPlacement.simulate(graph, p);
  } 
  catch(Exception ex) {
    println("Exception: " + ex.getMessage());
  }

  println("done!");
}

void draw() {
  background(255);

  translate(translateX, translateY);
  scale(scaleFactor);

  for (Edge e : graph.edgeSet()) {
    SimpleEdge se = (SimpleEdge)e;
    se.render();
  }

  for (Vertex v : graph.vertexSet()) {
    SimpleVertex sv = (SimpleVertex)v;
    sv.render();
  }

  surface.setTitle("Framerate: " + round(frameRate));
}


void mouseWheel(processing.event.MouseEvent e) {
  scaleFactor += e.getAmount() / 100;
  translateX -= e.getAmount() * mouseX / 100;
  translateY -= e.getAmount() * mouseY / 100;
}

boolean dragging = false;
PVector startPos = new PVector();

void mousePressed() {
  startPos = new PVector(mouseX, mouseY);
  dragging = true;
}

void mouseDragged() {
  if (dragging) {
    PVector endPos = new PVector(mouseX, mouseY);
    PVector delta = PVector.sub(endPos, startPos);
    translateX += delta.x;
    translateY += delta.y;

    startPos = new PVector(mouseX, mouseY);
  }
}

void mouseReleased() {
  dragging = false;
}

void keyPressed() {
  if (key == 'R') {
    scaleFactor = 1.0;
    translateX = 0.0;
    translateY = 0.0;
  }
}

void addEdge(SimpleVertex a, SimpleVertex b) {
  graph.addEdge(a, b, new SimpleEdge(a, b));
}
